//
//  Ball.m
//  Billard1
//
//  Created by Jean-Pierre IMBERT on 05/01/10.
//  Copyright 2010 JPI-Conseil. All rights reserved.
//

#import "Ball.h"

#pragma mark - Constants
const CGFloat speedDecreasingOnTable = 0.995;
const CGFloat speedDecreasingWhenShock = 0.9;
const CGFloat minInitialSpeed = 10.;
const CGFloat maxInitialSpeed = 20.;
const CGFloat minSpeedToStop = .3;
const NSTimeInterval timerInterval = 1./30.;
#define playersCount 2

#pragma mark - Class variables
AVAudioPlayer *player[playersCount];
NSUInteger nextPlayer;

#pragma mark - Utility functions
CGFloat randomSpeed (){
	srandomdev();
	CGFloat speed = minInitialSpeed+(maxInitialSpeed-minInitialSpeed)*random()/(CGFloat)INT_MAX;
	if (random()&01) {
		speed = -speed;
	}
	return speed;
}

#pragma mark -
@implementation Ball

#pragma mark - Méthodes dérivées de NSObject
+ (void)initialize {
	NSError *error;
	for (int i=0;i<playersCount;i++) {
		player[i] = [[AVAudioPlayer alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"ChocBille" ofType:@"wav" inDirectory:@"/"]] error:&error];
		[player[i] prepareToPlay];	
	}
	nextPlayer = 0;
}

- (id)init {
	if (self=[super init]){
		self.moveX = randomSpeed();
		self.moveY = randomSpeed();
	}
	return self;
}

#pragma mark - Accesseurs de propriétés
- (CGSize)size{
	if (self.ballView) return self.ballView.frame.size;
	return CGSizeZero;
}

- (CGPoint)center{
	if (self.ballView) return self.ballView.center;
	return CGPointZero;
}

- (void) setDelegate:(id <BallDelegate,NSObject>)aDelegate{
	if (aDelegate != _delegate) {
		_delegate = aDelegate;
		if (_delegate) {
			self.tableRect = [_delegate tableRect];
		}
	}
}

#pragma mark - Sonorisation des chocs
+ (void)shockNoise{
	[player[nextPlayer] play];
	nextPlayer = ++nextPlayer % playersCount;
	[player[nextPlayer] prepareToPlay];
}

#pragma mark - Fonctionnement de la boule
- (void) start {
	stop = NO;
	if ([self.delegate respondsToSelector:@selector(didStart:)]){
		[self.delegate didStart:self];
	}
	[self moveBall];
}

- (void) stop {
	stop = YES;
	if ([self.delegate respondsToSelector:@selector(didStop:)]){
		[self.delegate didStop:self];
	}
}

- (void) moveBall {

	// vérification si demande d'arrêt
	if (stop) return;
	
	// détection des bandes
	CGRect ballRect = self.ballView.frame;
	BOOL shockDetected = NO;
	if (CGRectGetMinX(ballRect)<CGRectGetMinX(self.tableRect)||CGRectGetMaxX(ballRect)>CGRectGetMaxX(self.tableRect)){
		self.moveX = -self.moveX;
		shockDetected = YES;
	}
	if (CGRectGetMinY(ballRect)<CGRectGetMinY(self.tableRect)||CGRectGetMaxY(ballRect)>CGRectGetMaxY(self.tableRect)){
		self.moveY = -self.moveY;
		shockDetected = YES;
	}
	
	// déplacement élémentaire
	CGPoint center = self.center;
	center.x += self.moveX;
	center.y += self.moveY;
	self.ballView.center = center;
	
	// décélération et sons
	if (shockDetected) {
		self.moveX *= speedDecreasingWhenShock;
		self.moveY *= speedDecreasingWhenShock;
		[Ball shockNoise];
	} else {
		self.moveX *= speedDecreasingOnTable;
		self.moveY *= speedDecreasingOnTable;
	}

	// Notification du délégué
	if ([self.delegate respondsToSelector:@selector(didMove:)]){
		[self.delegate didMove:self];
	}
	
	// détection de l'arrêt
	if (fabs(self.moveX)+fabs(self.moveY)>minSpeedToStop) {
		[self performSelector:@selector(moveBall) withObject:nil afterDelay:timerInterval];
	} else {
		stop = YES;
	}
}

@end
