//
//  BillardViewController.m
//  Billard
//
//  Created by Jean-Pierre IMBERT on 18/02/13.
//  Copyright (c) 2013 SkwalSoft. All rights reserved.
//

#import "BillardViewController.h"

const CGFloat speedDecreasingOnTable = 0.995;
const CGFloat speedDecreasingWhenShock = 0.9;
const CGFloat minInitialSpeed = 10.;
const CGFloat maxInitialSpeed = 20.;
const CGFloat minSpeedToStop = .3;
const NSTimeInterval timerInterval = 1./30.;
const float distancePrecision = 15.;
const float anglePrecision = 0.1;

CGFloat randomSpeed (){
	srandomdev();
	CGFloat speed = minInitialSpeed+(maxInitialSpeed-minInitialSpeed)*random()/(CGFloat)INT_MAX;
	if (random()&01) {
		speed = -speed;
	}
	return speed;
}

@interface BillardViewController ()

@end

@implementation BillardViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
	
	// initialisation des lecteurs audio
	NSError *error;
	player[0] = [[AVAudioPlayer alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"ChocBille" ofType:@"wav" inDirectory:@"/"]] error:&error];
	[player[0] prepareToPlay];
	player[1] = [[AVAudioPlayer alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"ChocBille" ofType:@"wav" inDirectory:@"/"]] error:&error];
	[player[1] prepareToPlay];
	nextPlayer = 0;
}

- (void) viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
	// vecteur vitesse initial
	moveX = randomSpeed();
	moveY = randomSpeed();
	
	// premier déplacement
    [(SnookerView *)self.view setLastPoint: self.ball.center];
    [(SnookerView *)self.view setDrawing:YES];
	[self moveBall];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) moveBall {
    
    // détection des bandes
    CGRect tableRect = self.view.bounds;
    CGRect ballRect = self.ball.frame;
	BOOL shockDetected = NO;
    if (CGRectGetMinX(ballRect)<CGRectGetMinX(tableRect)||
        CGRectGetMaxX(ballRect)>CGRectGetMaxX(tableRect)){
        moveX = -moveX;
		shockDetected = YES;
    }
    if (CGRectGetMinY(ballRect)<CGRectGetMinY(tableRect)||
        CGRectGetMaxY(ballRect)>CGRectGetMaxY(tableRect)){
        moveY = -moveY;
		shockDetected = YES;
    }
    
    // déplacement élémentaire
    CGPoint center = self.ball.center;
    center.x += moveX;
    center.y += moveY;
    self.ball.center = center;
    [(SnookerView*)self.view setLastPoint: center];
    
    // décélération et sons
	if (shockDetected) {
		moveX *= speedDecreasingWhenShock;
		moveY *= speedDecreasingWhenShock;
		[player[nextPlayer] play];
		nextPlayer = 1-nextPlayer;
		[player[nextPlayer] prepareToPlay];
	} else {
		moveX *= speedDecreasingOnTable;
		moveY *= speedDecreasingOnTable;
	}
	
    // détection de l'arrêt
	if (fabs(moveX)+fabs(moveY)>minSpeedToStop) {
        [self performSelector:@selector(moveBall) withObject:nil afterDelay:timerInterval];
    }
}

- (void)swipeMove:(CGRect)move
     withDuration:(NSTimeInterval)swipeDuration {
    // Détermination de la proximité de la boule
    // longueur de la chiquenaude
    CGFloat moveLength =
    sqrt(move.size.width*move.size.width+
         move.size.height*move.size.height);
    // Vecteur Origine-Boule et longueur
    CGSize ballVector =
    CGSizeMake(self.ball.center.x-move.origin.x,
               self.ball.center.y-move.origin.y);
    CGFloat ballLength =
    sqrt(ballVector.width*ballVector.width+
         ballVector.height*ballVector.height);
    // calcul du déterminant
    CGFloat det = (move.size.width*ballVector.height-
                   move.size.height*ballVector.width)/(moveLength*ballLength);
    // Modification de la vitesse de la boule
    if ((ballLength<moveLength+distancePrecision)&&
        (fabs(det)<anglePrecision)){
        moveX = move.size.width*timerInterval/swipeDuration;
        moveY = move.size.height*timerInterval/swipeDuration;
        // effacement de la table
        [(SnookerView*)self.view setDrawing:NO];
        [self.view setNeedsDisplay];
        // premier déplacement
        [(SnookerView*)self.view setLastPoint: self.ball.center];
        [(SnookerView*)self.view setDrawing:YES];
        [self moveBall];
    }
}

@end
