//
//  SnookerView.m
//  Billard
//
//  Created by Jean-Pierre IMBERT on 18/02/13.
//  Copyright (c) 2013 SkwalSoft. All rights reserved.
//

#import "SnookerView.h"

const float minSpeed = 300.;

@implementation SnookerView

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        _drawing = NO;
    }
    return self;
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect {
     if (self.drawing) {
         CGContextRef context = UIGraphicsGetCurrentContext();
         CGContextSetRGBStrokeColor(context, 1., 0.5, 0., 1.0);
         CGContextSetLineWidth(context, 3.);
         CGContextSetLineCap(context, kCGLineCapSquare);
         CGPoint segment[2] = {precedingLastPoint,self.lastPoint};
         CGContextStrokeLineSegments(context, segment, 2);
     }
}

-(void)setLastPoint:(CGPoint)aPoint {
    precedingLastPoint = _lastPoint;
    _lastPoint = aPoint;
    if (self.drawing) {
        [self setNeedsDisplayInRect:
         CGRectMake(precedingLastPoint.x, precedingLastPoint.y,
                    _lastPoint.x-precedingLastPoint.x,
                    _lastPoint.y-precedingLastPoint.y)];
    } else {
        precedingLastPoint = aPoint;
    }
}

- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event{
    lastTime = [[touches anyObject] timestamp];
}

- (void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event{
    UITouch * touch = [touches anyObject];
    NSTimeInterval currentTime = [touch timestamp];
    NSTimeInterval swipeDuration = currentTime-lastTime;
    lastTime = currentTime;
    CGPoint point1 = [touch previousLocationInView:self];
    CGPoint point2 = [touch locationInView:self];
    CGRect move = CGRectMake(point1.x, point1.y,
                             point2.x-point1.x, point2.y-point1.y);
    CGFloat speed = sqrt(move.size.width*move.size.width +
                         move.size.height*move.size.height)/swipeDuration;
    if (speed > minSpeed) {
        if ([self.delegate respondsToSelector:@selector(swipeMove:withDuration:)]) {
            [self.delegate swipeMove:move withDuration:swipeDuration];
        }
    }
}

- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event{
}

- (void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event{
}

@end
