//
//  AcceleroViewController.m
//  Accelero
//
//  Created by Jean-Pierre IMBERT on 22/02/13.
//  Copyright (c) 2013 SkwalSoft. All rights reserved.
//

#import "AcceleroViewController.h"
#import <CoreMotion/CoreMotion.h>

@interface AcceleroViewController ()
@property(strong,nonatomic) CMMotionManager *motionManager;
@property(strong,nonatomic) NSOperationQueue *queue;
@end

@implementation AcceleroViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    self.motionManager = [[CMMotionManager alloc] init];
    self.queue = [[NSOperationQueue alloc] init];
    // device delection
    self.selectedItem = 0;
    self.segControl.selectedSegmentIndex = self.selectedItem;
}

- (void)setSelectedItem:(NSInteger)selectedItem
{
    [self.motionManager stopAccelerometerUpdates];
    [self.motionManager stopDeviceMotionUpdates];
    _selectedItem = selectedItem;
    switch (selectedItem) {
        case 0:
            self.titleLabel.text = @"Accéléromètres";
            [self startAccelerometer];
            break;
            
        case 1:
            self.titleLabel.text = @"Gravité";
            [self startGravity];
            break;
            
        case 2:
            self.titleLabel.text = @"Accélérations";
            [self startUserAcceleration];
            break;
            
        default:
            break;
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)startAccelerometer
{
    if (self.motionManager.isAccelerometerAvailable) {
        self.motionManager.accelerometerUpdateInterval = 1.0 / 60.0;
        [self.motionManager startAccelerometerUpdatesToQueue:self.queue withHandler:^(CMAccelerometerData *data, NSError *error) {
            if (data) dispatch_async(dispatch_get_main_queue(), ^{
                self.xLabel.text = [NSString stringWithFormat:@"X : %f",data.acceleration.x];
                self.yLabel.text = [NSString stringWithFormat:@"Y : %f",data.acceleration.y];
                self.zLabel.text = [NSString stringWithFormat:@"Z : %f",data.acceleration.z];
                self.vert.vertLine = CGSizeMake(data.acceleration.x,data.acceleration.y);
            });
        }];
    }
}

- (void)startGravity
{
    if (self.motionManager.isDeviceMotionAvailable) {
        self.motionManager.deviceMotionUpdateInterval = 1.0 / 60.0;
        [self.motionManager startDeviceMotionUpdatesToQueue:self.queue withHandler:^(CMDeviceMotion *data, NSError *error) {
            if (data) dispatch_async(dispatch_get_main_queue(), ^{
                self.xLabel.text = [NSString stringWithFormat:@"X : %f",data.gravity.x];
                self.yLabel.text = [NSString stringWithFormat:@"Y : %f",data.gravity.y];
                self.zLabel.text = [NSString stringWithFormat:@"Z : %f",data.gravity.z];
                self.vert.vertLine = CGSizeMake(data.gravity.x,data.gravity.y);
            });
        }];
    }
}

- (void)startUserAcceleration
{
    if (self.motionManager.isDeviceMotionAvailable) {
        self.motionManager.deviceMotionUpdateInterval = 1.0 / 60.0;
        [self.motionManager startDeviceMotionUpdatesToQueue:self.queue withHandler:^(CMDeviceMotion *data, NSError *error) {
            if (data) dispatch_async(dispatch_get_main_queue(), ^{
                self.xLabel.text = [NSString stringWithFormat:@"X : %f",data.userAcceleration.x];
                self.yLabel.text = [NSString stringWithFormat:@"Y : %f",data.userAcceleration.y];
                self.zLabel.text = [NSString stringWithFormat:@"Z : %f",data.userAcceleration.z];
                self.vert.vertLine = CGSizeMake(data.userAcceleration.x,data.userAcceleration.y);
            });
        }];
    }
}

- (IBAction)changeSensor:(UISegmentedControl *)sender {
    self.selectedItem = sender.selectedSegmentIndex;
}

@end
