//
//  PenduModel.m
//  Pendu
//
//  Created by Jean-Pierre IMBERT on 26/02/13.
//  Copyright (c) 2013 SkwalSoft. All rights reserved.
//

#import "PenduModel.h"

#define minWordLength 6
#define maxWordLength 9
#define maxMistakes 11

@interface PenduModel () {
    __strong NSMutableString * _wordToGuess;
    __strong NSMutableString * _wordToDisplay;
}
@property (assign,nonatomic) NSInteger wordLength ;
@property (assign,nonatomic) NSInteger remainingLetters ;
@end

@implementation PenduModel

- (id) init {
    if (self = [super init]) {
        _wordToGuess = [[NSMutableString alloc] initWithCapacity:15];
        _wordToDisplay = [[NSMutableString alloc] initWithCapacity:15];
        [self reInit];
    }
    return self;
}

- (void) reInit {
    [_wordToGuess setString:@""];
    [_wordToDisplay setString:@"---------"];
    self.wordLength = 0;
    self.remainingLetters = self.wordLength;
    self.mistakes = 0;
}
- (void) reInitWithWord:(NSString *)word {
    [_wordToGuess setString:word];
    self.wordLength = [_wordToGuess length];
    [_wordToDisplay setString:@"---------"];
    [_wordToDisplay deleteCharactersInRange: NSMakeRange(self.wordLength,[self.wordToDisplay length]-self.wordLength)];
    self.remainingLetters = self.wordLength;
    self.mistakes = 0;
}

- (int) remainingFailed {
    return maxMistakes-self.mistakes;
}

- (BOOL) addALetter: (NSString *) letter {
    switch (self.wordLength) {
        case 0:
            [_wordToGuess setString:letter];
            self.wordLength++;
            [_wordToDisplay setString:@"-"];
            self.remainingLetters++;
            break;
        case maxWordLength:
            break;
        default:
            [_wordToGuess appendString:letter];
            self.wordLength++;
            [_wordToDisplay appendString:@"-"];
            self.remainingLetters++;
            break;
    }
    return (self.wordLength >= minWordLength);
}

- (BOOL) delLastLetter {
    if (self.wordLength > 0) {
        [_wordToGuess deleteCharactersInRange:NSMakeRange(self.wordLength-1, 1)];
        [_wordToDisplay deleteCharactersInRange:NSMakeRange(self.wordLength-1, 1)];
        self.wordLength--;
        self.remainingLetters--;
    }
    return (self.wordLength >= minWordLength);
}

- (BOOL) tryALetter: (NSString *) letter {
    BOOL keyOK = NO;
    unichar uniKey = [letter characterAtIndex:0];
    for (NSUInteger i=0; i<self.wordLength; i++) {
        if ([_wordToGuess characterAtIndex:i] == uniKey) {
            keyOK = YES;
            [_wordToDisplay replaceCharactersInRange:
             NSMakeRange(i,1) withString:letter];
            self.remainingLetters--;
        }
    }
    if (!keyOK) self.mistakes++;
    return (self.remainingLetters == 0) || (self.mistakes == maxMistakes);
}

- (BOOL) endedWithSuccess {
    return (self.wordLength != 0) && (self.remainingLetters == 0);
}

@end
