//
//  PenduViewController.m
//  Pendu
//
//  Created by Jean-Pierre IMBERT on 26/02/13.
//  Copyright (c) 2013 SkwalSoft. All rights reserved.
//

#import "PenduViewController.h"
#import "PenduStateInitial.h"
#import "KeyboardViewController.h"

@interface PenduViewController ()
@property (strong,nonatomic) KeyboardViewController * keyboard;
@property (assign,nonatomic) BOOL animating;
@end

@implementation PenduViewController

#pragma mark - View methods

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [self defineConstraints];
    [self addSpecificKeyboard];
    self.session = nil;
    self.connectedPeer = nil;
    [PenduState setViewController:self];
    self.currentState = [PenduStateInitial class];
    [self.currentState startApp];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) defineConstraints
{
    NSDictionary * views = @{@"topLabel":self.topLabel,@"imageView":self.imageView,@"middleLabel":self.middleLabel,
                             @"bottomLabel":self.bottomLabel,@"button":self.button,
                             @"activityIndicator":self.activityIndicator,@"keyboardView":self.keyboardView};
    // Remove All constraints by Interface Builder
    [self.view removeConstraints:[self.view constraints]];
    for (NSString * key in views) {
        [views[key] removeConstraints:[views[key] constraints]];
    }
    // Add vertical constraints
    NSArray * ca = [NSLayoutConstraint constraintsWithVisualFormat:
            @"V:|-[topLabel]-[imageView(100)]-(>=8)-[middleLabel(21)]-[bottomLabel]-[button]-[keyboardView]|"
                                                           options:NSLayoutFormatAlignAllCenterX
                                                           metrics:nil views:views];
    [self.view addConstraints:ca];
    // Add horizontal constraints
    NSLayoutConstraint * cn = [NSLayoutConstraint constraintWithItem:self.view
                                                           attribute:NSLayoutAttributeCenterX
                                                           relatedBy:NSLayoutRelationEqual
                                                              toItem:self.topLabel
                                                           attribute:NSLayoutAttributeCenterX
                                                          multiplier:1.0 constant:0.0];
    [self.view addConstraint:cn];
    // Add width constraints
    cn = [NSLayoutConstraint constraintWithItem:self.bottomLabel
                                      attribute:NSLayoutAttributeWidth
                                      relatedBy:NSLayoutRelationEqual
                                         toItem:self.view
                                      attribute:NSLayoutAttributeWidth
                                     multiplier:1.0 constant:-40.0];
    [self.view addConstraint:cn];
    // Add constraints for activityIndicator
    cn = [NSLayoutConstraint constraintWithItem:self.activityIndicator
                                      attribute:NSLayoutAttributeCenterX
                                      relatedBy:NSLayoutRelationEqual
                                         toItem:self.button
                                      attribute:NSLayoutAttributeCenterX
                                     multiplier:1.0 constant:0.0];
    [self.view addConstraint:cn];
    cn = [NSLayoutConstraint constraintWithItem:self.activityIndicator
                                      attribute:NSLayoutAttributeCenterY
                                      relatedBy:NSLayoutRelationEqual
                                         toItem:self.button
                                      attribute:NSLayoutAttributeCenterY
                                     multiplier:1.0 constant:0.0];
    [self.view addConstraint:cn];
}

- (void) addSpecificKeyboard
{
    self.keyboard = [[KeyboardViewController alloc] initWithNibName:nil bundle:nil];
    [self.keyboardView addSubview:self.keyboard.view];
    self.keyboard.view.translatesAutoresizingMaskIntoConstraints = NO;
    NSDictionary * views = @{@"keyboard":self.keyboard.view};
    NSArray * ca = [NSLayoutConstraint constraintsWithVisualFormat:@"H:|[keyboard(320)]|"
                                                           options:NSLayoutFormatAlignAllCenterX
                                                           metrics:nil views:views];
    [self.view addConstraints:ca];
    ca = [NSLayoutConstraint constraintsWithVisualFormat:@"V:|[keyboard(110)]|"
                                                 options:NSLayoutFormatAlignAllCenterX
                                                 metrics:nil views:views];
    [self.view addConstraints:ca];
}

#pragma mark - Accessors methods

- (void) setCurrentState:(Class)state {
    _currentState = state;
    [self.button removeTarget:nil action:@selector(actionOK) forControlEvents:UIControlEventTouchUpInside];
    [self.button addTarget:(id)self.currentState action:@selector(actionOK) forControlEvents:UIControlEventTouchUpInside];
    self.keyboard.delegate = (id)self.currentState;
}

- (void) setButtonEnabled:(BOOL)buttonEnabled
{
    self.button.enabled = buttonEnabled;
}

- (BOOL) buttonEnabled
{
    return self.button.enabled;
}

- (void) setDisplayedWord:(NSString *) word {
	self.middleLabel.text = word;
}

- (NSString *) displayedWord {
	return self.middleLabel.text;
}

- (void) setDisplayedMessage:(NSString *) message {
	self.bottomLabel.text = message;
}

- (NSString *) displayedMessage {
	return self.bottomLabel.text;
}

#pragma mark - Session management

- (void) startSession {
    if (!self.session) {
        self.session = [[GKSession alloc]
                        initWithSessionID:nil displayName:nil
                        sessionMode:GKSessionModePeer];
        self.session.delegate = self;
        [self.session setDataReceiveHandler:self withContext:nil];
        self.session.available = YES;
    }
}

- (void) clearSession {
    if (self.session) {
        [self.session disconnectFromAllPeers];
        self.session.available = NO;
        self.session.delegate = nil;
        [self.session setDataReceiveHandler:nil withContext:nil];
        self.connectedPeer = nil;
        self.session = nil;
    }
}

- (void) session:(GKSession *)aSession peer:(NSString *)peerID didChangeState:(GKPeerConnectionState)state {
    switch (state) {
        case GKPeerStateAvailable:
            if (!self.connectedPeer) {
                [aSession connectToPeer:peerID withTimeout:10.];
            }
            break;
        case GKPeerStateConnected:
            if (!self.connectedPeer) {
                self.connectedPeer = peerID;
                [self.currentState startGame:([self.connectedPeer compare:[aSession peerID]] > 0)];
            }
            break;
        case GKPeerStateDisconnected:
            if (self.connectedPeer == peerID) {
                self.connectedPeer = nil;
                [self.currentState connectionLost];
            }
            break;
        default:
            break;
    }
}

- (void) session:(GKSession *)aSession didReceiveConnectionRequestFromPeer:(NSString *)peerID {
    NSError * error;
    if (self.connectedPeer) {
        [aSession denyConnectionFromPeer:peerID];
    } else {
        [aSession acceptConnectionFromPeer:peerID error:&error];
    }
}

#pragma mark - Communication methods

- (void) sendString: (NSString *) string {
    NSError *error;
    [self.session sendData:[string
                            dataUsingEncoding:NSASCIIStringEncoding]
                   toPeers:[NSArray arrayWithObject:self.connectedPeer]
              withDataMode:GKSendDataReliable error:&error];
}

- (void) receiveData:(NSData *)data
            fromPeer:(NSString *)peer
           inSession: (GKSession *)aSession
             context:(void *)context {
    NSString * string = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
    [self.currentState receivedString:string];
}

#pragma mark - Appearance methods

- (void) presentWaitingForConnexionScreen {
    self.topLabel.text = @"Non connecté";
	[self startAnimation];
    self.middleLabel.hidden = YES;
    self.bottomLabel.hidden = NO;
    self.bottomLabel.text = @"Recherche d'un autre joueur";
    [self.activityIndicator startAnimating];
    self.button.hidden = YES;
    self.keyboard.hidden = YES;
}

- (void) presentWaitingForWordToGuessScreen {
    self.topLabel.text = [NSString stringWithFormat:@"Connecté à %@",[self.session displayNameForPeer:self.connectedPeer]];
	[self startAnimation];
    self.middleLabel.hidden = YES;
    self.bottomLabel.hidden = NO;
    self.bottomLabel.text = [NSString stringWithFormat:@"Attendez le choix du mot à deviner"];
    [self.activityIndicator startAnimating];
    self.button.hidden = YES;
    self.keyboard.hidden = YES;
}

- (void) presentWordEnteringScreen {
    self.topLabel.text = [NSString stringWithFormat:@"Connecté à %@",[self.session displayNameForPeer:self.connectedPeer]];
	[self startAnimation];
    self.middleLabel.hidden = NO;
    self.middleLabel.text = @"_________";
    self.bottomLabel.text = @"Saisissez un mot de 6 à 9 lettres";
    [self.activityIndicator stopAnimating];
    self.button.hidden = NO;
    self.button.enabled = NO;
    self.keyboard.hidden = NO;
    self.keyboard.autoDisable = NO;
    self.keyboard.withBackspace = YES;
    [self.keyboard enableAll];
}

- (void) presentWaitingForGuessScreen {
    self.topLabel.text = [NSString stringWithFormat:@"Connecté à %@",[self.session displayNameForPeer:self.connectedPeer]];
	[self stopAnimation];
	[self setImageNumber:[NSNumber numberWithInt:0]];
    self.middleLabel.hidden = NO;
    self.middleLabel.text = self.model.wordToDisplay;
    self.bottomLabel.text = [NSString stringWithFormat:@"Le mot à deviner est '%@'. Il reste %d echecs possibles",self.model.wordToGuess, self.model.remainingFailed];
    [self.activityIndicator stopAnimating];
    self.button.hidden = YES;
    self.keyboard.hidden = YES;
}

- (void) presentGuessingWordScreen {
    self.topLabel.text = [NSString stringWithFormat:@"Connecté à %@", [self.session displayNameForPeer:self.connectedPeer]];
	[self stopAnimation];
	[self setImageNumber:[NSNumber numberWithInt:0]];
    self.middleLabel.hidden = NO;
    self.middleLabel.text = self.model.wordToDisplay;
    self.bottomLabel.text = [NSString stringWithFormat:@"Devinez le mot. Il vous reste %d échecs possibles.", self.model.remainingFailed];
    [self.activityIndicator stopAnimating];
    self.button.hidden = YES;
    self.keyboard.hidden = NO;
    self.keyboard.autoDisable = YES;
    self.keyboard.withBackspace = NO;
    [self.keyboard enableAll];
}

- (void) presentNewGameScreenWithButton:(BOOL)withButton {
	self.topLabel.text = [NSString stringWithFormat:@"Connecté à %@",[self.session displayNameForPeer:self.connectedPeer]];
	self.middleLabel.hidden = NO;
	if (self.model.endedWithSuccess) {
		self.middleLabel.text = @"GAGNÉ";
		self.bottomLabel.text  = [NSString stringWithFormat:@"Le mot %@ a été deviné.",self.model.wordToGuess];
	} else {
		self.middleLabel.text = @"PERDU !";
		self.bottomLabel.text  = [NSString stringWithFormat:@"Le mot %@ n'a pas été deviné.",self.model.wordToGuess];
	}
	if (withButton) {
		self.bottomLabel.text  = [self.bottomLabel.text  stringByAppendingString:@" À vous de faire deviner un mot."];
		[self.activityIndicator stopAnimating];
		self.button.hidden = NO;
	} else {
		self.bottomLabel.text  = [self.bottomLabel.text  stringByAppendingString:@" Attendez votre tour pour deviner un mot."];
		[self.activityIndicator startAnimating];
		self.button.hidden = YES;
	}
	self.keyboard.hidden = YES;
}

#pragma mark Images methods

- (void) setImageNumber:(NSNumber *) imageNumber {
	int number = [imageNumber intValue];
	UIImage * image = [UIImage imageNamed:[NSString stringWithFormat:@"Pendu%2.2d.png",number]];
	if (!image) {
		number = 0;
		image = [UIImage imageNamed:[NSString stringWithFormat:@"Pendu%2.2d.png",number]];
	}
	self.imageView.image = image;
	if (self.animating) [self performSelector:@selector(setImageNumber:) withObject:[NSNumber numberWithInt:number+1] afterDelay:1.0];
}

- (void) startAnimation {
	if (!self.animating) {
		[self setImageNumber:[NSNumber numberWithInt:0]];
		[self performSelector:@selector(setImageNumber:) withObject:[NSNumber numberWithInt:1] afterDelay:1.0];
		self.animating = YES;
	}
}

- (void) stopAnimation {
	if (self.animating) [NSObject cancelPreviousPerformRequestsWithTarget:self];
	self.animating = NO;
}

@end
